<?php

namespace Drupal\site_verify\Controller;

use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\site_verify\Service\SiteVerifyService;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for Site Verify module routes.
 */
class SiteVerifyController implements ContainerInjectionInterface {

  use AutowireTrait;
  use StringTranslationTrait;

  /**
   * Constructs a new SiteVerifyController object.
   */
  public function __construct(
    protected Connection $database,
    protected RedirectDestinationInterface $redirectDestination,
    protected RouteBuilderInterface $routeBuilder,
    #[Autowire(service: 'site_verify_service')]
    protected SiteVerifyService $siteVerifyService,
    TranslationInterface $string_translation,
  ) {
    $this->stringTranslation = $string_translation;
  }

  /**
   * Controller content callback: Verifications List page.
   *
   * @return string
   *   Render Array
   */
  public function verificationsListPage() {
    // @todo Why is this called every time the list is shown? In the admin form
    // RouteBuilderInterface::setRebuildNeeded() is called on change.
    $this->routeBuilder->rebuild();

    $engines = $this->siteVerifyService->siteVerifyGetEngines();
    $destination = $this->redirectDestination->getAsArray();

    $header = [
      ['data' => $this->t('Engine'), 'field' => 'engine'],
      ['data' => $this->t('Meta tag'), 'field' => 'meta'],
      ['data' => $this->t('File'), 'field' => 'file'],
      ['data' => $this->t('Operations')],
    ];

    $verifications = $this->database->select('site_verify', 'sv')
      ->fields('sv')
      ->execute();

    $rows = [];
    foreach ($verifications as $verification) {
      $row = ['data' => []];
      $row['data'][] = $engines[$verification->engine]['name'];
      $row['data'][] = $verification->meta ? $this->t('Yes') : $this->t('No');
      $row['data'][] = $verification->file ? Link::fromTextAndUrl($verification->file, Url::fromRoute('site_verify.' . $verification->file)) : $this->t('None');
      $operations = [];
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'url' => Url::fromRoute('site_verify.verification_edit', ['site_verify' => $verification->svid]),
        'query' => $destination,
      ];
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('site_verify.verification_delete', ['site_verify' => $verification->svid]),
        'query' => $destination,
      ];
      $row['data']['operations'] = [
        'data' => [
          '#theme' => 'links',
          '#links' => $operations,
          '#attributes' => ['class' => ['links', 'inline']],
        ],
      ];
      $rows[] = $row;
    }

    $build['verification_table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No verifications available. <a href="@add">Add verification</a>.', ['@add' => Url::fromRoute('site_verify.verification_add')->toString()]),
    ];
    return $build;
  }

  /**
   * Controller content callback: Verifications File content.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response containing the Verification File content.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   In the event file is set but engine does not support files.
   */
  public function verificationsFileContent($svid) {
    $verification = $this->siteVerifyService->siteVerifyLoad($svid);
    if ($verification['file'] && $verification['engine']['file_contents']) {
      $response = new Response(
        $verification['file_contents'],
        200,
        ['Content-Type' => 'text/plain']
      );
      return $response;
    }

    // Not a file verification.
    throw new NotFoundHttpException();
  }

}
