<?php

namespace Drupal\site_verify\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\site_verify\Service\SiteVerifyService;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Builds the form to delete a forum term.
 */
class SiteVerifyDeleteForm extends ConfirmFormBase implements ContainerInjectionInterface {

  use AutowireTrait;

  /**
   * Constructs a new SiteVerifyController object.
   */
  public function __construct(
    protected Connection $database,
    protected RouteBuilderInterface $routeBuilder,
    #[Autowire(service: 'site_verify_service')]
    protected SiteVerifyService $siteVerifyService,
    TranslationInterface $string_translation,
    #[Autowire(service: 'logger.channel.site_verify')]
    protected LoggerChannelInterface $logger,
  ) {
    $this->stringTranslation = $string_translation;
  }

  /**
   * Verification record to confirm deletion of.
   *
   * @var string
   */
  protected $siteVerify = NULL;

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['site_verify.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'site_verify_confirm_delete';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    if (!empty($this->siteVerify)) {
      $record = $this->siteVerifyService->siteVerifyLoad($this->siteVerify);
      return $this->t(
        'Are you sure you want to delete the site verification %label?', [
          '%label' => $record['engine']['name'],
        ]
      );
    }

    // Function must still return a TranslatableMarkup even if missing.
    return $this->t('Are you sure you want to delete teh site verification?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('site_verify.verifications_list');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $site_verify = NULL) {

    $this->siteVerify = $site_verify;
    $record = $this->siteVerifyService->siteVerifyLoad($this->siteVerify);

    $form = parent::buildForm($form, $form_state);

    $form['record'] = [
      '#type' => 'value',
      '#value' => $record,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $record = $form_state->getValue('record');
    $this->database->delete('site_verify')
      ->condition('svid', $record['svid'])
      ->execute();
    $this->messenger()->addStatus($this->t('Verification for %engine has been deleted.', [
      '%engine' => $record['engine']['name'],
    ]));
    $this->logger->notice($this->t('Verification for %engine deleted.', [
      '%engine' => $record['engine']['name'],
    ]));
    $form_state->setRedirect('site_verify.verifications_list');

    // Set the menu to be rebuilt.
    $this->routeBuilder->setRebuildNeeded();
  }

}
