<?php

declare(strict_types=1);

namespace Drupal\Tests\site_verify\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\TestFileCreationTrait;

/**
 * Tests functionality of the admin forms and verification modes.
 *
 * @group site_verify
 */
class SiteVerifyTest extends BrowserTestBase {

  use TestFileCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected bool $usesSuperUserAccessPolicy = FALSE;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'file',
    'node',
    'system',
    'site_verify',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with "administer site verify" permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * The nid of a non-front page node.
   *
   * @var int
   */
  protected $otherNodeId;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // An admin user who can manage site verity verifications.
    $this->adminUser = $this->drupalCreateUser(['administer site verify']);

    // Add a content type and set a node as front page.
    $this->drupalCreateContentType(['type' => 'front']);
    $test_node = $this->drupalCreateNode([
      'type' => 'front',
      'title' => $this->randomMachineName(8),
    ]);
    $this->config('system.site')
      ->set('page.front', '/node/' . $test_node->id())
      ->save();

    // Add an extra node which is not the front page.
    $this->otherNodeId = $this->drupalCreateNode([
      'type' => 'front',
      'title' => $this->randomMachineName(8),
    ])->id();
  }

  /**
   * Test Generate Password alter user forms.
   */
  public function testAdminAndFunction(): void {
    // Normal user cannot access the admin page.
    $this->drupalGet('admin/config/search/verifications');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('admin/config/search/verifications/add');
    $this->assertSession()->statusCodeEquals(403);

    // Admin user can access verifications. And there are presently none.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/config/search/verifications');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('No verifications available.');

    // Add a meta verification.
    $this->drupalGet('admin/config/search/verifications/add');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm(['engine' => 'google'], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'meta' => '<meta name="google-site-verification" content="NbwaW3WIDp_SPsSsfl78Ive7F34-znm9lxLJXjuWNGE" />',
    ], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Verification saved.');

    // File based verification using fields entry.
    $this->drupalGet('admin/config/search/verifications/add');
    $this->submitForm(['engine' => 'custom'], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'file' => 'googleb5741169901f6c20.html',
      'file_contents' => 'google-site-verification: googleb5741169901f6c20.html',
    ], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Verification saved.');

    // File based verification using file upload.
    $this->drupalGet('admin/config/search/verifications/add');
    $this->submitForm(['engine' => 'custom'], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    // Magic number: 1024 = 16 * 64, the size of the smallest auto-generated
    // file in TestFileCreationTrait::getTestFiles().
    $test_file = current($this->getTestFiles('text', 1024));
    /** @var \Drupal\Core\File\FileSystemInterface $file_system */
    $file_system = \Drupal::service('file_system');
    $this->submitForm([
      'files[file_upload]' => $file_system->realpath($test_file->uri),
    ], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Verification saved.');

    // Logout as admin and check access as anonymous user.
    $this->drupalLogout();

    // File based verification.
    $this->drupalGet('googleb5741169901f6c20.html');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('google-site-verification: googleb5741169901f6c20.html');

    // Uploaded file based verification. Contents are a randomly generated very
    // long string. Check for part of it.
    $this->drupalGet($test_file->filename);
    $this->assertSession()->statusCodeEquals(200);
    // Magic number: 64, the length of one line of text in auto-generated file
    // in TestFileCreationTrait::getTestFiles().
    $test_file_contents_subset = substr(file_get_contents($test_file->uri), 0, 64);
    $this->assertSession()->responseContains($test_file_contents_subset);

    // Meta tag based verification appears on front page.
    $this->drupalGet('<front>');
    $this->assertSession()->statusCodeEquals(200);
    $xpath = $this->xpath("//meta[@name='google-site-verification']");
    $this->assertCount(1, $xpath);
    $this->assertEquals(
      'NbwaW3WIDp_SPsSsfl78Ive7F34-znm9lxLJXjuWNGE',
      $xpath[0]->getAttribute('content')
    );

    // Meta tag does not appear on other page.
    $this->drupalGet('node/' . $this->otherNodeId);
    $this->assertSession()->statusCodeEquals(200);
    $xpath = $this->xpath("//meta[@name='google-site-verification']");
    $this->assertCount(0, $xpath);

    // Login as admin to delete verifications.
    $this->drupalLogin($this->adminUser);

    // Get a list of the verifications.
    $verifications = \Drupal::database()->select('site_verify', 'site_verify')
      ->fields('site_verify', ['svid', 'meta', 'file'])
      ->execute()
      ->fetchAllAssoc('svid');

    // Wipe each in turn and verify they have been removed.
    foreach ($verifications as $verification) {
      $this->drupalGet("admin/config/search/verifications/{$verification->svid}/delete");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains('Are you sure you want to delete the site verification');
      $this->submitForm([], 'Delete');

      if ($verification->meta) {
        $this->drupalGet('<front>');
        $this->assertSession()->statusCodeEquals(200);
        $xpath = $this->xpath("//meta[@name='google-site-verification']");
        $this->assertCount(0, $xpath);
      }
      elseif ($verification->file) {
        $this->drupalGet($verification->file);
        $this->assertSession()->statusCodeEquals(404);
      }
      else {
        throw new \Exception('Unhandled verification type.');
      }
    }
  }

}
